package cn.com.qmth.examcloud.support.fss;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.bind.DatatypeConverter;
import java.io.File;
import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.MessageDigest;
import java.util.Base64;

public class FssHelper {

    private static final Logger log = LoggerFactory.getLogger(FssHelper.class);

    /**
     * 获取文件的最终完整访问路径（兼容旧数据）
     *
     * @param filePath 文件存储路径，格式示例：
     *                 https://xxx.xxx/abc/123.jpg
     *                 upyun-1://abc/123.jpg
     *                 aliyun-1://abc/123.jpg
     *                 aliyun-2://abc/123.jpg
     *                 /abc/123.jpg
     *                 abc/123.jpg
     * @return 最终访问地址，如: https://xxx.xxx/abc/123.jpg
     */
    public static String finalFileUrl(String filePath) {
        if (StringUtils.isEmpty(filePath)) {
            return "";
        }

        if (filePath.toLowerCase().startsWith("http")) {
            return filePath;
        }

        filePath = filePath.replace("upyun-1://", "").replace("aliyun-1://", "").replace("aliyun-2://", "");

        if (filePath.startsWith("/")) {
            return FssProperty.FSS_URL_PREFIX + filePath;
        }

        return FssProperty.FSS_URL_PREFIX + FssProperty.FSS_SEPARATOR + filePath;
    }

    /**
     * 修正文件存储路径，若包含域名则去掉（兼容旧数据）
     *
     * @param filePath 文件存储路径，格式示例：
     *                 https://xxx.xxx/abc/123.jpg
     *                 http://127.0.0.1:8080/abc/123.jpg
     *                 upyun-1://abc/123.jpg
     *                 aliyun-1://abc/123.jpg
     *                 aliyun-2://abc/123.jpg
     *                 /abc/123.jpg
     *                 abc/123.jpg
     * @return 标准文件存储路径，如：/abc/123.jpg
     */
    public static String fixFilePath(String filePath) {
        if (StringUtils.isEmpty(filePath)) {
            return "";
        }

        if (filePath.toLowerCase().startsWith("http")) {
            try {
                URL url = new URL(filePath);
                return url.getPath();
            } catch (MalformedURLException e) {
                log.error(filePath, e.getMessage(), e);
                throw new RuntimeException("文件路径格式错误！");
            }
        }

        filePath = filePath.replace("upyun-1://", "").replace("aliyun-1://", "").replace("aliyun-2://", "");

        if (!filePath.startsWith("/")) {
            return FssProperty.FSS_SEPARATOR + filePath;
        }
        return filePath;
    }

    /**
     * 创建文件目录
     */
    public static boolean makeDirs(String dirPath) {
        if (StringUtils.isEmpty(dirPath)) {
            return false;
        }

        File dir = new File(dirPath);
        if (!dir.exists()) {
            return dir.mkdirs();
        }

        return true;
    }

    /**
     * 获取文件名(包含后缀名)
     */
    public static String getFileName(String filePath) {
        if (filePath != null) {
            filePath = filePath.replace("\\", "/");

            int index = filePath.lastIndexOf("/");
            if (index >= 0) {
                return filePath.substring(index + 1);
            }
        }

        return filePath;
    }

    /**
     * 获取文件MD5值
     */
    public static String getFileMD5(byte[] fileBytes) {
        return DigestUtils.md5Hex(fileBytes);
    }

    /**
     * 获取文件的MD5值
     */
    public static String getFileMD5(File file) {
        if (file == null) {
            throw new RuntimeException("文件不能为空！");
        }

        if (!file.isFile()) {
            throw new RuntimeException("文件类型错误！");
        }

        try (FileInputStream fis = new FileInputStream(file);) {
            MessageDigest md = MessageDigest.getInstance("MD5");

            int len;
            byte[] bytes = new byte[4096];
            while ((len = fis.read(bytes)) != -1) {
                md.update(bytes, 0, len);
            }

            return new String(Hex.encodeHex(md.digest()));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("获取文件的MD5值失败！");
        }
    }

    public static String md5ToBase64(String md5) {
        if (StringUtils.isBlank(md5)) {
            throw new IllegalArgumentException("Invalid md5 parameter");
        }

        if (md5.length() == 32) {
            return Base64.getEncoder().encodeToString(DatatypeConverter.parseHexBinary(md5));
        } else if (md5.length() == 24) {
            // base64格式
            return md5;
        }
        throw new IllegalArgumentException("Invalid md5 length");
    }

}
